#!/usr/bin/env python

import os
from pyglet import image
from pyglet import window
from pyglet import clock
from pyglet import font
from pyglet import graphics
from pyglet import gl
from math import sqrt
from math import fabs
from math import ceil
from math import floor
from math import sin
from math import cos
from math import pi
import random
from common import *
from wind import *
from leaf import *
from blob import *

class LeafGameWindow(window.Window):

	def __init__(self, *args, **kwargs):
		self.max_leaves = 50
		self.max_wind_particles = 15
		self.blob_creation_range = 20
		window.Window.__init__(self, *args, **kwargs)
		self.set_mouse_visible(False)
		self.init_sprites()
		self.wind_map = Windmap(self.width, self.height, 15)
		#start at one, to avoid division by zero
		self.last_leaf = 0
		self.total_time = 0
		
		self.draw_wind_map = True
		
		#sound
		self.bach = load_music("Air_(Bach).ogg")
		self.restart_music = self.bach.duration + 3
		self.music = self.bach.play()
		
		#bg image
		self.bg = Sprite("bg.png", **kwargs)
		self.help_screen = Sprite("help.png", **kwargs)

	def init_sprites(self):
		self.leaves = []
		self.blobs = []
		self.wind_particles = []
		
		self.wind_particle_image = load_image("wind.png")
		self.leaf_image = [load_image("leaf1.png"), load_image("leaf2.png"), load_image("leaf3.png"), load_image("leaf4.png"), load_image("leaf5.png"), load_image("leaf6.png")]
		
		self.player = Windplayer(self.width - 150, 10, x=100,y=100)
		
		while len(self.wind_particles) < self.max_wind_particles:#Creates all Windparticles
			self.wind_particles.append(Windparticle(self.player, self.wind_particle_image, x=random.randint(0, self.width), y=random.randint(0, self.height)))

	def main_loop(self):
		ft = font.load('Arial', 28)
		#fps_text = font.Text(ft, y=10)
		leaf_text = font.Text(ft, y=10)

		clock.set_fps_limit(30)
		
		self.running = False

		while not self.has_exit:
			self.dispatch_events()
			self.clear()
			
			#looping is really wierd
			if self.restart_music < 0:
				self.restart_music = self.bach.duration + 3
				print self.restart_music
				self.music.seek(0)
				self.music.play()
			
			if self.running:
				self.update()
			self.draw()

			#Tick the clock
			dt = clock.tick()
			self.last_leaf += dt
			self.total_time += dt
			self.restart_music -= dt
			#Gets fps and draw it
			#fps_text.text = ("fps: %d") % (clock.get_fps())
			#fps_text.draw()
			leaf_text.text = ("Leaves: %d/50") % len(self.leaves)
			leaf_text.draw()
			self.flip()

	def update(self):

		to_remove = []
		
		if len(self.leaves) >= self.max_leaves:
			self.running = False
		
		#add leaf
		if self.last_leaf > 300 / (self.total_time + 100):
			#print (300 / (self.total_time + 100))
			self.last_leaf = 0
			self.create_leaf()
		
		for sprite in self.leaves:
			sprite.update(self.wind_map)
			
			#Is it not registered in a blob?
			if ( not sprite.in_blob):
				# Is any blob covering this leaf?
				for blob in self.blobs:
					if ( sprite.distance(blob) < blob.radius ):
						sprite.attach_blob(blob, True)
						break
				else:
					# Is this leaf near another unblobbed leaf?
					close_leaves = [sprite]
					for leaf in self.leaves:
						if ( not leaf.in_blob and sprite.distance(leaf) < self.blob_creation_range and leaf is not sprite ):
							close_leaves.append(leaf)
					if (len(close_leaves) > 1):
						self.blobs.append(Blob(close_leaves, self.blob_creation_range, self))
			
			#Is it dead?
			if (sprite.dead):
				to_remove.append(sprite)
				
		#Remove dead sprites
		for sprite in to_remove:
			self.leaves.remove(sprite)
		
		#Update blobs
		for blob in self.blobs:
			blob.update()
		
		self.player.update(self.wind_map)
		#Is it dead?
		"""leaf_hit = self.player.collide_once(self.leaves)
		if (leaf_hit is not None):
			self.player.dead = True
			self.has_exit = True
		"""
		for i in range(len(self.wind_particles)):# i is used to avoid double collision prevention
			self.wind_particles[i].update(self.player, i, self.wind_particles, self.wind_map)
		
		self.wind_map.update()

	def draw(self):
		gl.glEnable(gl.GL_BLEND)
		gl.glBlendFunc(gl.GL_SRC_ALPHA, gl.GL_ONE_MINUS_SRC_ALPHA)
		gl.glDisable(gl.GL_DEPTH_TEST)
		
		
		self.bg.draw()
		
		if not self.running:
			self.help_screen.draw()
		
		if self.draw_wind_map:
			gl.glColor4f(0, 1, 0, 0.5)
			self.wind_map.draw()
			gl.glColor4f(1, 1, 1, 1)
		for sprite in self.leaves:
			sprite.draw()
		for sprite in self.blobs:
			sprite.draw()
		for sprite in self.wind_particles:
			sprite.draw()
		self.player.draw()

	def create_leaf(self):
		if (len(self.leaves) < self.max_leaves):
			self.leaves.append(Leaf(self.leaf_image[random.randint(0,5)]
				, x=random.randint(5, self.width-5) , y=random.randint(5, self.height-5)))
	
	def kill_blob(self, blob):
		try:
			self.blobs.remove(blob)
		except:
			pass
	
	def kill_leaf(self, leaf):
		try:
			self.leaves.remove(leaf)
		except:
			pass

	"""******************************************
	Event Handlers
	*********************************************"""
	def on_mouse_motion(self, x, y, dx, dy):
		self.player.x = x
		self.player.y = y
	
	def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
		self.player.x = x
		self.player.y = y
	
	def on_mouse_press(self, x, y, button, modifiers):
		if button == window.mouse.RIGHT:
			self.draw_wind_map = not self.draw_wind_map
		elif not self.running:
			self.running = True
			self.player.points = 0
			self.total_time = 0
			#remove all leaves
			self.leaves = []
			self.blobs = []
			#creates some leaves
			self.create_leaf()
			self.create_leaf()
			self.create_leaf()
			self.create_leaf()


if __name__ == "__main__":
	# Someone is launching this directly
	space = LeafGameWindow(width=800, height=600)
	space.main_loop()

